@@ -26,6 +26,7 @@ import static org.apache.fury.codegen.Code.LiteralValue.TrueLiteral;
 import static org.apache.fury.codegen.CodeGenerator.alignIndent;
 import static org.apache.fury.codegen.CodeGenerator.appendNewlineIfNeeded;
 import static org.apache.fury.codegen.CodeGenerator.indent;
+import static org.apache.fury.collection.Collections.ofArrayList;
 import static org.apache.fury.type.TypeUtils.BOOLEAN_TYPE;
 import static org.apache.fury.type.TypeUtils.CLASS_TYPE;
 import static org.apache.fury.type.TypeUtils.ITERABLE_TYPE;
@@ -37,6 +38,7 @@ import static org.apache.fury.type.TypeUtils.PRIMITIVE_LONG_TYPE;
 import static org.apache.fury.type.TypeUtils.PRIMITIVE_SHORT_TYPE;
 import static org.apache.fury.type.TypeUtils.PRIMITIVE_VOID_TYPE;
 import static org.apache.fury.type.TypeUtils.STRING_TYPE;
+import static org.apache.fury.type.TypeUtils.VOID_TYPE;
 import static org.apache.fury.type.TypeUtils.boxedType;
 import static org.apache.fury.type.TypeUtils.defaultValue;
 import static org.apache.fury.type.TypeUtils.getArrayType;
@@ -52,6 +54,7 @@ import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.List;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
@@ -79,7 +82,6 @@ import org.apache.fury.util.function.SerializableTriFunction;
  * tree into subtrees, we can replace dependent expression fields in subtree root nodes with {@link
  * Reference}.
  */
-@SuppressWarnings("UnstableApiUsage")
 public interface Expression {
 
   /**
@@ -131,9 +133,37 @@ public interface Expression {
   // ####################### Expressions #######################
   // ###########################################################
 
-  abstract class Inlineable implements Expression {
+  abstract class AbstractExpression implements Expression {
+    protected final transient List<Expression> inputs;
+
+    protected AbstractExpression(Expression input) {
+      this.inputs = Collections.singletonList(input);
+    }
+
+    protected AbstractExpression(Expression[] inputs) {
+      this.inputs = Arrays.asList(inputs);
+    }
+
+    protected AbstractExpression(List<Expression> inputs) {
+      this.inputs = inputs;
+    }
+  }
+
+  abstract class Inlineable extends AbstractExpression {
     protected boolean inlineCall = false;
 
+    protected Inlineable(Expression input) {
+      super(input);
+    }
+
+    protected Inlineable(Expression[] inputs) {
+      super(inputs);
+    }
+
+    protected Inlineable(List<Expression> inputs) {
+      super(inputs);
+    }
+
     public Inlineable inline() {
       return inline(true);
     }
@@ -149,6 +179,10 @@ public interface Expression {
     // set to others to get a more context-dependent variable name.
     public String valuePrefix = "value";
 
+    protected ValueExpression(Expression[] inputs) {
+      super(inputs);
+    }
+
     public String isNullPrefix() {
       return "is" + StringUtils.capitalize(valuePrefix) + "Null";
     }
@@ -158,7 +192,7 @@ public interface Expression {
    * A ListExpression is a list of expressions. Use last expression's type/nullable/value/IsNull to
    * represent ListExpression's type/nullable/value/IsNull.
    */
-  class ListExpression implements Expression {
+  class ListExpression extends AbstractExpression {
     private final List<Expression> expressions;
     private Expression last;
 
@@ -167,6 +201,7 @@ public interface Expression {
     }
 
     public ListExpression(List<Expression> expressions) {
+      super(expressions);
       this.expressions = expressions;
       if (!this.expressions.isEmpty()) {
         this.last = this.expressions.get(this.expressions.size() - 1);
@@ -239,7 +274,57 @@ public interface Expression {
     }
   }
 
-  class Literal implements Expression {
+  class Variable extends AbstractExpression {
+    private final String namePrefix;
+    private Expression from;
+
+    public Variable(String namePrefix, Expression from) {
+      super(from);
+      this.namePrefix = namePrefix;
+      this.from = from;
+    }
+
+    @Override
+    public TypeRef<?> type() {
+      return from.type();
+    }
+
+    @Override
+    public boolean nullable() {
+      return from.nullable();
+    }
+
+    @Override
+    public ExprCode doGenCode(CodegenContext ctx) {
+      StringBuilder codeBuilder = new StringBuilder();
+      ExprCode targetExprCode = from.genCode(ctx);
+      if (StringUtils.isNotBlank(targetExprCode.code())) {
+        codeBuilder.append(targetExprCode.code()).append('\n');
+      }
+      String name = ctx.newName(namePrefix);
+      String decl =
+          StringUtils.format(
+              "${type} ${name} = ${from};",
+              "type",
+              ctx.type(type()),
+              "name",
+              name,
+              "from",
+              targetExprCode.value());
+      codeBuilder.append(decl);
+      return new ExprCode(
+          codeBuilder.toString(),
+          targetExprCode.isNull(),
+          Code.variable(type().getRawType(), name));
+    }
+
+    @Override
+    public String toString() {
+      return String.format("%s %s = %s;", type(), namePrefix, from);
+    }
+  }
+
+  class Literal extends AbstractExpression {
     public static final Literal True = new Literal(true, PRIMITIVE_BOOLEAN_TYPE);
     public static final Literal False = new Literal(false, PRIMITIVE_BOOLEAN_TYPE);
 
@@ -247,15 +332,21 @@ public interface Expression {
     private final TypeRef<?> type;
 
     public Literal(String value) {
+      super(new Expression[0]);
       this.value = value;
       this.type = STRING_TYPE;
     }
 
     public Literal(Object value, TypeRef<?> type) {
+      super(new Expression[0]);
       this.value = value;
       this.type = type;
     }
 
+    public static Literal ofBoolean(boolean v) {
+      return v ? True : False;
+    }
+
     public static Literal ofByte(short v) {
       return new Literal(v, PRIMITIVE_BYTE_TYPE);
     }
@@ -362,7 +453,7 @@ public interface Expression {
     }
   }
 
-  class Null implements Expression {
+  class Null extends AbstractExpression {
     private TypeRef<?> type;
     private final boolean typedNull;
 
@@ -371,6 +462,7 @@ public interface Expression {
     }
 
     public Null(TypeRef<?> type, boolean typedNull) {
+      super(new Expression[0]);
       this.type = type;
       this.typedNull = typedNull;
     }
@@ -395,8 +487,8 @@ public interface Expression {
   }
 
   /** A Reference is a variable/field that can be accessed in the expression's CodegenContext. */
-  class Reference implements Expression {
-    private final String name;
+  class Reference extends AbstractExpression {
+    private String name;
     private final TypeRef<?> type;
     private final boolean nullable;
     private final boolean fieldRef;
@@ -414,6 +506,7 @@ public interface Expression {
     }
 
     public Reference(String name, TypeRef<?> type, boolean nullable, boolean fieldRef) {
+      super(new Expression[0]);
       this.name = name;
       this.type = type;
       this.nullable = nullable;
@@ -424,6 +517,10 @@ public interface Expression {
       return new Reference(name, type, false, true);
     }
 
+    public void setName(String name) {
+      this.name = name;
+    }
+
     @Override
     public TypeRef<?> type() {
       return type;
@@ -462,7 +559,11 @@ public interface Expression {
     }
   }
 
-  class Empty implements Expression {
+  class Empty extends AbstractExpression {
+
+    public Empty() {
+      super(new Expression[0]);
+    }
 
     @Override
     public TypeRef<?> type() {
@@ -475,10 +576,11 @@ public interface Expression {
     }
   }
 
-  class Block implements Expression {
+  class Block extends AbstractExpression {
     private final String code;
 
     public Block(String code) {
+      super(new Expression[0]);
       this.code = code;
     }
 
@@ -493,10 +595,11 @@ public interface Expression {
     }
   }
 
-  class ForceEvaluate implements Expression {
+  class ForceEvaluate extends AbstractExpression {
     private Expression expression;
 
     public ForceEvaluate(Expression expression) {
+      super(expression);
       this.expression = expression;
     }
 
@@ -537,6 +640,7 @@ public interface Expression {
         TypeRef<?> type,
         boolean fieldNullable,
         boolean inline) {
+      super(targetObject);
       Preconditions.checkArgument(type != null);
       this.targetObject = targetObject;
       this.fieldName = fieldName;
@@ -628,12 +732,13 @@ public interface Expression {
     }
   }
 
-  class SetField implements Expression {
+  class SetField extends AbstractExpression {
     private Expression targetObject;
     private final String fieldName;
     private Expression fieldValue;
 
     public SetField(Expression targetObject, String fieldName, Expression fieldValue) {
+      super(targetObject);
       this.targetObject = targetObject;
       this.fieldName = fieldName;
       this.fieldValue = fieldValue;
@@ -679,12 +784,15 @@ public interface Expression {
    * An expression to set up a stub in expression tree, so later tree building can replace it with
    * other expression.
    */
-  class ReplaceStub implements Expression {
+  class ReplaceStub extends AbstractExpression {
     private Expression targetObject;
 
-    public ReplaceStub() {}
+    public ReplaceStub() {
+      super(new Expression[0]);
+    }
 
     public ReplaceStub(Expression targetObject) {
+      super(targetObject);
       this.targetObject = targetObject;
     }
 
@@ -728,6 +836,7 @@ public interface Expression {
         String castedValueNamePrefix,
         boolean inline,
         boolean ignoreUpcast) {
+      super(targetObject);
       this.targetObject = targetObject;
       this.type = type;
       this.castedValueNamePrefix = castedValueNamePrefix;
@@ -791,13 +900,14 @@ public interface Expression {
     }
 
     @Override
-    public Inlineable inline(boolean inlineCall) {
+    public Cast inline(boolean inlineCall) {
       if (!inlineCall) {
-        if (targetObject instanceof Inlineable) {
+        AbstractExpression expr = (AbstractExpression) targetObject;
+        if (expr instanceof Inlineable && expr.inputs.size() > 1) {
           ((Inlineable) targetObject).inlineCall = false;
         }
       }
-      return super.inline(inlineCall);
+      return (Cast) super.inline(inlineCall);
     }
 
     @Override
@@ -827,6 +937,7 @@ public interface Expression {
         boolean returnNullable,
         boolean inline,
         boolean needTryCatch) {
+      super(arguments);
       this.functionName = functionName;
       this.type = type;
       this.arguments = arguments;
@@ -1185,7 +1296,7 @@ public interface Expression {
     }
   }
 
-  class NewInstance implements Expression {
+  class NewInstance extends AbstractExpression {
     private TypeRef<?> type;
     private final Class<?> rawType;
     private String unknownClassName;
@@ -1212,6 +1323,7 @@ public interface Expression {
     }
 
     private NewInstance(TypeRef<?> type, List<Expression> arguments, Expression outerPointer) {
+      super(ofArrayList(outerPointer, arguments));
       this.type = type;
       rawType = getRawType(type);
       this.outerPointer = outerPointer;
@@ -1333,7 +1445,7 @@ public interface Expression {
     }
   }
 
-  class NewArray implements Expression {
+  class NewArray extends AbstractExpression {
     private TypeRef<?> type;
     private Expression[] elements;
 
@@ -1345,6 +1457,7 @@ public interface Expression {
     private Expression dims;
 
     public NewArray(Class<?> elemType, Expression dim) {
+      super(dim);
       this.numDimensions = 1;
       this.elemType = elemType;
       this.dim = dim;
@@ -1359,6 +1472,7 @@ public interface Expression {
      * @param dims an int[] represent dims
      */
     public NewArray(Class<?> elemType, int numDimensions, Expression dims) {
+      super(dims);
       this.numDimensions = numDimensions;
       this.elemType = elemType;
       this.dims = dims;
@@ -1371,6 +1485,7 @@ public interface Expression {
     }
 
     public NewArray(TypeRef<?> type, Expression... elements) {
+      super(elements);
       this.type = type;
       this.elements = elements;
       this.numDimensions = 1;
@@ -1482,12 +1597,13 @@ public interface Expression {
     }
   }
 
-  class AssignArrayElem implements Expression {
+  class AssignArrayElem extends AbstractExpression {
     private Expression targetArray;
     private Expression value;
     private Expression[] indexes;
 
     public AssignArrayElem(Expression targetArray, Expression value, Expression... indexes) {
+      super(ofArrayList(targetArray, value, indexes));
       this.targetArray = targetArray;
       this.value = value;
       this.indexes = indexes;
@@ -1529,7 +1645,7 @@ public interface Expression {
     }
   }
 
-  class If implements Expression {
+  class If extends AbstractExpression {
     private Expression predicate;
     private Expression trueExpr;
     private Expression falseExpr;
@@ -1537,6 +1653,7 @@ public interface Expression {
     private boolean nullable;
 
     public If(Expression predicate, Expression trueExpr) {
+      super(new Expression[] {predicate, trueExpr});
       this.predicate = predicate;
       this.trueExpr = trueExpr;
       this.nullable = false;
@@ -1549,6 +1666,7 @@ public interface Expression {
         Expression falseExpr,
         boolean nullable,
         TypeRef<?> type) {
+      super(new Expression[] {predicate, trueExpr, falseExpr});
       this.predicate = predicate;
       this.trueExpr = trueExpr;
       this.falseExpr = falseExpr;
@@ -1563,6 +1681,7 @@ public interface Expression {
 
     /** if predicate eval to null, take predicate as false. */
     public If(Expression predicate, Expression trueExpr, Expression falseExpr) {
+      super(new Expression[] {predicate, trueExpr, falseExpr});
       this.predicate = predicate;
       this.trueExpr = trueExpr;
       this.falseExpr = falseExpr;
@@ -1633,7 +1752,6 @@ public interface Expression {
       }
       if (!PRIMITIVE_VOID_TYPE.equals(type.unwrap())) {
         ExprCode falseEval = falseExpr.doGenCode(ctx);
-        Preconditions.checkArgument(trueEval.isNull() != null || falseEval.isNull() != null);
         Preconditions.checkNotNull(trueEval.value());
         Preconditions.checkNotNull(falseEval.value());
         Class<?> rawType = getRawType(type);
@@ -1643,6 +1761,7 @@ public interface Expression {
         codeBuilder.append(String.format("%s %s;\n", ctx.type(type), value));
         String ifCode;
         if (nullable) {
+          Preconditions.checkArgument(trueEval.isNull() != null || falseEval.isNull() != null);
           codeBuilder.append(String.format("boolean %s = false;\n", isNull));
           String trueEvalIsNull;
           if (trueEval.isNull() == null) {
@@ -1759,10 +1878,11 @@ public interface Expression {
     }
   }
 
-  class IsNull implements Expression {
+  class IsNull extends AbstractExpression {
     private Expression expr;
 
     public IsNull(Expression expr) {
+      super(expr);
       this.expr = expr;
     }
 
@@ -1789,10 +1909,11 @@ public interface Expression {
     }
   }
 
-  class Not implements Expression {
+  class Not extends AbstractExpression {
     private Expression target;
 
     public Not(Expression target) {
+      super(target);
       this.target = target;
       Preconditions.checkArgument(
           target.type() == PRIMITIVE_BOOLEAN_TYPE || target.type() == BOOLEAN_TYPE);
@@ -1839,6 +1960,7 @@ public interface Expression {
 
     protected BinaryOperator(
         boolean inline, String operator, Expression left, Expression right, TypeRef<?> t) {
+      super(new Expression[] {left, right});
       this.inlineCall = inline;
       this.operator = operator;
       this.left = left;
@@ -2002,7 +2124,17 @@ public interface Expression {
     }
 
     public LogicalAnd(boolean inline, Expression left, Expression right) {
-      super(inline, "&", left, right);
+      super(inline, "&&", left, right);
+    }
+  }
+
+  class LogicalOr extends LogicalOperator {
+    public LogicalOr(Expression left, Expression right) {
+      super(true, "||", left, right);
+    }
+
+    public LogicalOr(boolean inline, Expression left, Expression right) {
+      super(inline, "||", left, right);
     }
   }
 
@@ -2017,8 +2149,8 @@ public interface Expression {
    *     return val
    * </pre>
    */
-  class While implements Expression {
-    private final BinaryOperator predicate;
+  class While extends AbstractExpression {
+    private final Expression predicate;
     private Expression action;
     private Expression[] cutPoints;
 
@@ -2027,11 +2159,15 @@ public interface Expression {
      *
      * @param predicate predicate must be inline.
      */
-    public While(BinaryOperator predicate, Expression action) {
+    public While(Expression predicate, Expression action) {
       this(predicate, action, new Expression[0]);
     }
 
-    public While(BinaryOperator predicate, SerializableSupplier<Expression> action) {
+    /**
+     * Use lambda to create a new context, and by capturing variables, we can make the codegen of
+     * thoese variable expressions happen before while loop.
+     */
+    public While(Expression predicate, SerializableSupplier<Expression> action) {
       this(
           predicate,
           action.get(),
@@ -2039,11 +2175,11 @@ public interface Expression {
               .toArray(new Expression[0]));
     }
 
-    public While(BinaryOperator predicate, Expression action, Expression[] cutPoints) {
+    public While(Expression predicate, Expression action, Expression[] cutPoints) {
+      super(ofArrayList(predicate, action, cutPoints));
       this.predicate = predicate;
       this.action = action;
       this.cutPoints = cutPoints;
-      Preconditions.checkArgument(predicate.inlineCall, predicate);
     }
 
     @Override
@@ -2084,11 +2220,10 @@ public interface Expression {
     }
   }
 
-  class ForEach implements Expression {
+  class ForEach extends AbstractExpression {
     private Expression inputObject;
 
-    @ClosureVisitable
-    private final SerializableBiFunction<Expression, Expression, Expression> action;
+    @ClosureVisitable final SerializableBiFunction<Expression, Expression, Expression> action;
 
     private final TypeRef<?> elementType;
 
@@ -2098,6 +2233,7 @@ public interface Expression {
      */
     public ForEach(
         Expression inputObject, SerializableBiFunction<Expression, Expression, Expression> action) {
+      super(inputObject);
       this.inputObject = inputObject;
       this.action = action;
       TypeRef elementType;
@@ -2113,6 +2249,7 @@ public interface Expression {
         Expression inputObject,
         TypeRef<?> beanType,
         SerializableBiFunction<Expression, Expression, Expression> action) {
+      super(inputObject);
       this.inputObject = inputObject;
       this.action = action;
       this.elementType = beanType;
@@ -2208,7 +2345,7 @@ public interface Expression {
     }
   }
 
-  class ZipForEach implements Expression {
+  class ZipForEach extends AbstractExpression {
     private Expression left;
     private Expression right;
 
@@ -2219,6 +2356,7 @@ public interface Expression {
         Expression left,
         Expression right,
         SerializableTriFunction<Expression, Expression, Expression, Expression> action) {
+      super(new Expression[] {left, right});
       this.left = left;
       this.right = right;
       this.action = action;
@@ -2348,22 +2486,27 @@ public interface Expression {
     }
   }
 
-  class ForLoop implements Expression {
+  class ForLoop extends AbstractExpression {
     public Expression start;
     public Expression end;
     public Expression step;
-
-    @ClosureVisitable public final SerializableFunction<Expression, Expression> action;
+    private final Class<?> maxType;
+    private final Reference iref;
+    public Expression loopAction;
 
     public ForLoop(
         Expression start,
         Expression end,
         Expression step,
         SerializableFunction<Expression, Expression> action) {
+      super(new Expression[] {start, end, step});
       this.start = start;
       this.end = end;
       this.step = step;
-      this.action = action;
+      this.maxType = maxType(getRawType(start.type()), getRawType(end.type()));
+      Preconditions.checkArgument(maxType.isPrimitive());
+      iref = new Reference(String.valueOf(System.identityHashCode(this)), TypeRef.of(maxType));
+      this.loopAction = action.apply(iref);
     }
 
     @Override
@@ -2373,12 +2516,9 @@ public interface Expression {
 
     @Override
     public ExprCode doGenCode(CodegenContext ctx) {
-      Class<?> maxType = maxType(getRawType(start.type()), getRawType(end.type()));
-      Preconditions.checkArgument(maxType.isPrimitive());
       StringBuilder codeBuilder = new StringBuilder();
       String i = ctx.newName("i");
-      Reference iref = new Reference(i, TypeRef.of(maxType));
-      Expression loopAction = action.apply(iref);
+      iref.setName(i);
       ExprCode startExprCode = start.genCode(ctx);
       ExprCode endExprCode = end.genCode(ctx);
       ExprCode stepExprCode = step.genCode(ctx);
@@ -2420,12 +2560,13 @@ public interface Expression {
    * <p>Since value is declared to be {@code List<elementType>}, no need to cast in other expression
    * that need List
    */
-  class ListFromIterable implements Expression {
+  class ListFromIterable extends AbstractExpression {
     private final TypeRef elementType;
     private Expression inputObject;
     private final TypeRef<?> type;
 
     public ListFromIterable(Expression inputObject) {
+      super(inputObject);
       this.inputObject = inputObject;
       Preconditions.checkArgument(
           getRawType(inputObject.type()) == Iterable.class,
@@ -2482,20 +2623,28 @@ public interface Expression {
     }
   }
 
-  class Return implements Expression {
+  class Return extends AbstractExpression {
     private Expression expression;
 
+    public Return() {
+      super(new Expression[0]);
+    }
+
     public Return(Expression expression) {
+      super(expression);
       this.expression = expression;
     }
 
     @Override
     public TypeRef<?> type() {
-      return expression.type();
+      return expression == null ? VOID_TYPE : expression.type();
     }
 
     @Override
     public ExprCode doGenCode(CodegenContext ctx) {
+      if (expression == null) {
+        return new ExprCode("return;", null, null);
+      }
       StringBuilder codeBuilder = new StringBuilder();
       ExprCode targetExprCode = expression.genCode(ctx);
       if (StringUtils.isNotBlank(targetExprCode.code())) {
@@ -2507,17 +2656,39 @@ public interface Expression {
 
     @Override
     public String toString() {
-      return String.format("return %s", expression);
+      return expression != null ? String.format("return %s", expression) : "return;";
     }
   }
 
-  class Assign implements Expression {
-    private Expression from;
-    private Expression to;
+  class Break extends AbstractExpression {
+    public Break() {
+      super(new Expression[0]);
+    }
 
-    public Assign(Expression from, Expression to) {
-      this.from = from;
+    @Override
+    public TypeRef<?> type() {
+      return PRIMITIVE_VOID_TYPE;
+    }
+
+    @Override
+    public ExprCode doGenCode(CodegenContext ctx) {
+      return new ExprCode("break;", null, null);
+    }
+
+    @Override
+    public String toString() {
+      return "break;";
+    }
+  }
+
+  class Assign extends AbstractExpression {
+    private Expression to;
+    private Expression from;
+
+    public Assign(Expression to, Expression from) {
+      super(new Expression[] {to, from});
       this.to = to;
+      this.from = from;
     }
 
     @Override
@@ -2539,7 +2710,7 @@ public interface Expression {
               });
       String assign =
           StringUtils.format(
-              "${from} = ${to};", "from", fromExprCode.value(), "to", toExprCode.value());
+              "${to} = ${from};", "from", fromExprCode.value(), "to", toExprCode.value());
       codeBuilder.append(assign);
       return new ExprCode(codeBuilder.toString(), null, null);
     }
