@@ -62,13 +62,6 @@ public class MapSerializer<T extends Map> extends AbstractMapSerializer<T> {
     return (T) map;
   }
 
-  @Override
-  public T read(MemoryBuffer buffer) {
-    Map map = newMap(buffer);
-    readElements(buffer, getAndClearNumElements(), map);
-    return onMapRead(map);
-  }
-
   @Override
   public T onMapRead(Map map) {
     return (T) map;
