@@ -634,4 +634,16 @@ public class MemoryBufferTest {
       assertEquals(buf.readVarUint36Small(), 0); // overflow
     }
   }
+
+  @Test
+  public void testReadBytesAsInt64() {
+    for (MemoryBuffer buffer :
+        new MemoryBuffer[] {
+          MemoryUtils.buffer(16), MemoryUtils.wrap(ByteBuffer.allocateDirect(32)),
+        }) {
+      buffer.writeByte(10);
+      buffer.writeByte(20);
+      assertEquals(buffer.readBytesAsInt64(2), (20 << 8) | 10);
+    }
+  }
 }
